from __future__ import annotations

import datetime
from contextvars import ContextVar
from typing import Any, Type
from uuid import uuid4

import iso8601
from iso8601 import ParseError


def uid() -> str:
    return str(uuid4()).replace("-", "")


_uid = uid


def utcnow() -> datetime.datetime:
    return datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc)


def str_to_datetime_if_parseable(value: str) -> datetime.datetime | str:
    if len(value) < 8 or value.count("-") != 2:
        return value
    try:
        ret = iso8601.parse_date(value)
    except (ParseError, ValueError, TypeError):
        ret = value
    return ret


class Undefined:
    pass


def resolve_undefined_type_context(
    context_var: ContextVar,
    new_type: Any,
    parameter: Any | None | Type[Undefined] = Undefined,
) -> Any:
    if parameter is Undefined:
        parameter = context_var.get()
    elif parameter is None:
        parameter = new_type()
    return parameter



from __future__ import annotations

import datetime
from typing import Dict, List, Any

from pydantic import BaseModel, Field, ConfigDict

from pydantic_db_backend_common.utils import utcnow, uid


class BackendModel(BaseModel):
    uid: str | None = Field(default_factory=uid)
    revision: str | None = None
    version: int | None = 1
    created_time: datetime.datetime | None = Field(default_factory=utcnow)
    updated_time: datetime.datetime | None = Field(default_factory=utcnow)



class PaginationParameterModel(BaseModel):
    skip: int | None = None
    limit: int | None = None
    sort: str | None = None
    view: str | None = None
    filter: dict | None = None


class PaginationResponseModel(BaseModel):
    data: Dict[str, dict] = {}
    skip: int | None = Field(None, nullable=True)
    limit: int | None = Field(None, nullable=True)
    max_results: int | None = None
    sort: str | None = None
    view: str | None = None
    ids: List[str] | None = []


class FindResultModel(BaseModel):
    data: List[Any] | None = []
    max_results: int | None = None


class AggregationModel(BaseModel):
    pipeline: List[dict] = []
    use_facet: bool | None = False


class CustomAggregationModel(BaseModel):
    before_tail: List[dict] | None = None
